@@ -5,4 +5,4 @@ from .xxzz import XXZZQubit
 from .repetition import RepetitionQubit
 from .xzzx import XZZXQubit
 from .circ import TopologicalRegister, TopologicalCircuit
-from .constants import *
+from .qubit_types import str2qtype
