@@ -6,15 +6,14 @@ import math
 from abc import abstractmethod, ABCMeta
 from itertools import combinations
 from typing import Tuple, List, Dict, Optional, Union, cast
-from IPython.display import Image, display
 
-import pydot
 import retworkx as rx
+from retworkx.visualization import mpl_draw
 import numpy as np
+from matplotlib import figure, axes
 import matplotlib.pyplot as plt
 from mpl_toolkits.mplot3d import Axes3D
 from qtcodes.fitters.base import TopologicalDecoder
-from qtcodes.fitters.graph_utils import GraphUtils
 
 TQubit = Tuple[float, float, float]  # (time,row,column) ==> (t,i,j)
 TQubitLoc = Tuple[float, float]  # (row,column) ==> (i,j)
@@ -35,15 +34,25 @@ class LatticeDecoder(TopologicalDecoder[TQubit], metaclass=ABCMeta):
 
     def __init__(self, params: Dict) -> None:
         super().__init__(params)
-        if "d" not in self.params or "T" not in self.params:
-            raise ValueError("Please include d and T in params.")
+        self._params_validation()
+
         for syndrome_graph_key in self.syndrome_graph_keys:
             self.S[syndrome_graph_key] = rx.PyGraph(multigraph=False)
             self.node_map[syndrome_graph_key] = {}
         self.virtual = self._specify_virtual()
-        self.encoder = self.encoder_type(params.copy())
+        self._encoder = None
         self._make_syndrome_graph()
 
+    @property
+    def encoder(self):
+        if self._encoder is None:
+            self._encoder = self.encoder_type(self.params.copy())
+        return self._encoder
+
+    def _params_validation(self):
+        if "d" not in self.params or "T" not in self.params:
+            raise ValueError("Please include d and T in params.")
+
     @abstractmethod
     def _specify_virtual(self) -> Dict[str, List[TQubit]]:
         """
@@ -109,7 +118,7 @@ class LatticeDecoder(TopologicalDecoder[TQubit], metaclass=ABCMeta):
             self.S[syndrome_graph_key]
         )
 
-        num_shortest_paths: Dict[int, List[int]] = {}
+        num_shortest_paths: Dict[int, Dict[int, int]] = {}
         for source, target in combinations(nodes, 2):
             if (
                 source in self.virtual[syndrome_graph_key]
@@ -146,9 +155,9 @@ class LatticeDecoder(TopologicalDecoder[TQubit], metaclass=ABCMeta):
         a: TQubit,
         b: TQubit,
         syndrome_graph_key: str,
-        num_shortest_paths: Dict[int, List[int]],
+        num_shortest_paths: Dict[int, Dict[int, int]],
         shortest_distance_mat: np.ndarray,
-    ) -> Tuple[int, Dict[int, List[int]]]:
+    ) -> Tuple[int, Dict[int, Dict[int, int]]]:
         """
         Calculate the number of shortest error paths (degeneracy)
         that link two syndrome nodes through both space and time.
@@ -164,7 +173,7 @@ class LatticeDecoder(TopologicalDecoder[TQubit], metaclass=ABCMeta):
             a (tuple): Starting or ending syndrome node (degeneracy is symmetric)
             b (tuple): Ending or starting syndrome node (degeneracy is symmetric)
             syndrome_graph_key (str): specifies the syndrome graph (e.g. "X")
-            num_shortest_paths (Dict[int, List[int]]):
+            num_shortest_paths (Dict[int, Dict[int, int]]):
                 key: source node
                 val:
                     val[i] corresponds to number of shortest paths
@@ -176,7 +185,7 @@ class LatticeDecoder(TopologicalDecoder[TQubit], metaclass=ABCMeta):
 
         Returns:
             num (int): Number of degenerate shortest paths matching this syndrome pair
-            num_shortest_paths (Dict[int, List[int]]]): updated num_shortest_paths
+            num_shortest_paths (Dict[int, Dict[int, int]]]): updated num_shortest_paths
         """
         a_indx = self.node_map[syndrome_graph_key][a]
         b_indx = self.node_map[syndrome_graph_key][b]
@@ -212,8 +221,8 @@ class LatticeDecoder(TopologicalDecoder[TQubit], metaclass=ABCMeta):
         a_indx: int,
         b_indx: int,
         syndrome_graph_key: str,
-        num_shortest_paths: Dict[int, List[int]],
-    ) -> Tuple[int, Dict[int, List[int]]]:
+        num_shortest_paths: Dict[int, Dict[int, int]],
+    ) -> Tuple[int, Dict[int, Dict[int, int]]]:
         """Helper to calculate the number of shortest error paths that link two syndrome nodes
         through both space and time.
 
@@ -221,7 +230,7 @@ class LatticeDecoder(TopologicalDecoder[TQubit], metaclass=ABCMeta):
             a_indx (int): Indx of one node
             b_indx (int): Indx of another node
             syndrome_graph_key (str): specifies the syndrome graph (e.g. "X")
-            num_shortest_paths (Dict[int, List[int]]):
+            num_shortest_paths (Dict[int, Dict[int, int]]):
                 key: source node
                 val:
                     val[i] corresponds to number of shortest paths
@@ -229,15 +238,15 @@ class LatticeDecoder(TopologicalDecoder[TQubit], metaclass=ABCMeta):
                     This saves some computations if already calculated.
         Returns:
             num (int): Number of degenerate shortest paths matching this syndrome pair
-            num_shortest_paths (Dict[int, List[int]]]): updated num_shortest_paths
+            num_shortest_paths (Dict[int, Dict[int, int]]]): updated num_shortest_paths
         """
         if a_indx in num_shortest_paths.keys():
             return num_shortest_paths[a_indx][b_indx], num_shortest_paths
         elif b_indx in num_shortest_paths.keys():
             return num_shortest_paths[b_indx][a_indx], num_shortest_paths
         else:
-            num_shortest_paths[a_indx] = GraphUtils.num_shortest_paths(
-                self.S[syndrome_graph_key], a_indx
+            num_shortest_paths[a_indx] = dict(
+                rx.num_shortest_paths_unweighted(self.S[syndrome_graph_key], a_indx)
             )
             return num_shortest_paths[a_indx][b_indx], num_shortest_paths
 
@@ -409,35 +418,59 @@ class LatticeDecoder(TopologicalDecoder[TQubit], metaclass=ABCMeta):
         """
         return self.encoder.parse_readout(readout_string, readout_type)
 
-    def draw(self, graph: rx.PyGraph) -> None:
+    def draw(
+        self,
+        graph: rx.PyGraph,
+        dpi: Optional[int] = None,
+        node_size: Optional[int] = None,
+        font_size: Optional[float] = None,
+        show: Optional[bool] = True,
+    ) -> Tuple[figure.Figure, axes.Axes]:
         """
         Plots 2D graphs in IPython/Jupyter.
 
         Args:
             graph (rx.PyGraph): graph to be plotted
+            dpi (int): dpi used for Figure. Defaults to dynamically sized value based on node count.
+            node_size (int): size of node used for `mpl_draw`. Defaults to dynamically sized value based on node count.
+            font_size (float): font size used for `mpl_draw`. Defaults to dynamically sized value based on node count.
+            show (bool): whether to display the plot automatically. Defaults to True.
+
+        Returns:
+            (figure, axes): A matplotlib Figure and Axes object
         """
         if self.params["T"] > 1:
             self.draw3D(graph)
             return
-        pdot = pydot.graph_from_dot_data(
-            graph.to_dot(
-                edge_attr=lambda e: {
-                    "label": str(round(float(e), 3)),
-                    "fontsize": "8",
-                },
-                node_attr=lambda n: {
-                    "label": str(n),
-                    "pos": f'"{n[2]},{-1.0*n[1]}!"',
-                    "fontsize": "8",
-                    "color": "black",
-                    "fillcolor": "lightblue",
-                    "style": "filled",
-                },
-                graph_attr={"outputorder": "edgesfirst"},
-            )
-        )[0]
-        plt_image = Image(pdot.create_png(prog="fdp"))
-        display(plt_image)
+
+        node_count = len(graph.nodes())
+        scale = 5 / (math.sqrt(node_count) if node_count > 0 else 5)
+        dpi = dpi if dpi is not None else 150 / scale
+        node_size = node_size if node_size is not None else 1750 * scale
+        font_size = font_size if font_size is not None else 6 * scale
+
+        positions = {}
+        for idx, node in enumerate(graph.nodes()):
+            positions[idx] = [node[2], -node[1]]
+
+        fig = plt.figure(dpi=dpi)
+        ax = fig.subplots()
+        mpl_draw(
+            graph,
+            ax=ax,
+            with_labels=True,
+            pos=positions,
+            labels=str,
+            edge_labels=lambda e: str(np.around(e, 3)),
+            node_size=node_size,
+            node_color="lightblue",
+            font_size=font_size,
+            alpha=0.8,
+        )
+        fig.tight_layout()
+        if not show:
+            plt.close(fig)
+        return (fig, ax)
 
     def draw3D(self, graph: rx.PyGraph, angle: Optional[List[float]] = None) -> None:
         """Plots a graph with edge labels in 3D.
@@ -464,7 +497,8 @@ class LatticeDecoder(TopologicalDecoder[TQubit], metaclass=ABCMeta):
         # 3D network plot
         with plt.style.context(("ggplot")):
             fig = plt.figure(figsize=(20, 14))
-            ax = Axes3D(fig)
+            ax = Axes3D(fig, auto_add_to_figure=False)
+            fig.add_axes(ax)
 
             # Loop on the nodes and look up in pos dictionary to extract the x,y,z coordinates of each node
             for node in graph.nodes():
